@@ -2,10 +2,12 @@
 
 #include <atomic>
 #include <bit>
+#include <list>
 #include <map>
 #include <optional>
 #include <vector>
 #include <memory>
+#include <set>
 #include <unordered_set>
 #include <unordered_map>
 
@@ -14,9 +16,15 @@
 #include <pl/api.hpp>
 
 #include <pl/core/errors/runtime_errors.hpp>
+#include <pl/core/ast/ast_node_type_appilication.hpp>
 
-#include <fmt/format.h>
+#include <fmt/core.h>
 #include <fmt/ranges.h>
+#include <pl/pattern_language.hpp>
+
+namespace pl {
+    class PatternLanguage;
+}
 
 namespace pl::ptrn {
 
@@ -69,6 +77,13 @@ namespace pl::core {
         };
 
         struct Scope {
+            Scope(const std::shared_ptr<pl::ptrn::Pattern>& parentPattern,
+                std::vector<std::shared_ptr<pl::ptrn::Pattern>>* scopePatterns,
+                size_t heapSize)
+                : parent(parentPattern),
+                scope(scopePatterns),
+                heapStartSize(heapSize) {}
+
             std::shared_ptr<ptrn::Pattern> parent;
             std::vector<std::shared_ptr<ptrn::Pattern>> *scope;
             std::optional<ParameterPack> parameterPack;
@@ -85,6 +100,13 @@ namespace pl::core {
             ~UpdateHandler();
 
             Evaluator *evaluator;
+            const ast::ASTNode *node = nullptr;
+            u64 offset = 0;
+        };
+
+        struct StackTrace {
+            std::unique_ptr<ast::ASTNode> node;
+            u64 cursorAddress;
         };
 
         void pushScope(const std::shared_ptr<ptrn::Pattern> &parent, std::vector<std::shared_ptr<ptrn::Pattern>> &scope);
@@ -114,31 +136,62 @@ namespace pl::core {
             return this->m_scopes.size() == 1;
         }
 
-        [[nodiscard]] const std::vector<std::unique_ptr<ast::ASTNode>>& getCallStack() const {
+        [[nodiscard]] const std::vector<StackTrace>& getCallStack() const {
             return this->m_callStack;
         }
 
         void pushTemplateParameters() {
-            if (this->m_templateParameters.empty())
                 this->m_templateParameters.emplace_back();
-            else
-                this->m_templateParameters.push_back(this->m_templateParameters.back());
         }
 
         void popTemplateParameters() {
             this->m_templateParameters.pop_back();
         }
 
-        [[nodiscard]] const std::vector<std::shared_ptr<ptrn::Pattern>>& getTemplateParameters() {
+        [[nodiscard]] const std::vector<std::shared_ptr<ptrn::Pattern>>& getTemplateParameters() const {
             return this->m_templateParameters.back();
         }
 
+        [[nodiscard]] std::vector<std::shared_ptr<ptrn::Pattern>>& getTemplateParameters() {
+            return this->m_templateParameters.back();
+        }
+
+        void pushTypeTemplateParameters() {
+            this->m_typeTemplateParameters.emplace_back();
+        }
+
+        void popTypeTemplateParameters() {
+            this->m_typeTemplateParameters.pop_back();
+        }
+
+        [[nodiscard]] const std::vector<std::shared_ptr<ast::ASTNode>>& getTypeTemplateParameters() const {
+            return this->m_typeTemplateParameters.back();
+        }
+
+        [[nodiscard]] std::vector<std::shared_ptr<ast::ASTNode>>& getTypeTemplateParameters() {
+            return this->m_typeTemplateParameters.back();
+        }
+
+        void setCurrentTemplateArguments(std::vector<std::unique_ptr<ast::ASTNode>> &&args) {
+            this->m_currentTemplateArguments = std::move(args);
+        }
+
+        [[nodiscard]] const std::vector<std::unique_ptr<ast::ASTNode>>& getCurrentTemplateArguments() const {
+            return this->m_currentTemplateArguments;
+        }
+
+        [[nodiscard]] std::vector<std::unique_ptr<ast::ASTNode>>& getCurrentTemplateArguments() {
+            return this->m_currentTemplateArguments;
+        }
+
         void pushSectionId(u64 id);
         void popSectionId();
         [[nodiscard]] u64 getSectionId() const;
+        [[nodiscard]] u64 getUserSectionId() const;
         [[nodiscard]] u64 createSection(const std::string &name);
         void removeSection(u64 id);
         [[nodiscard]] std::vector<u8>& getSection(u64 id);
+        [[nodiscard]] u64 getSectionSize(u64 id);
         [[nodiscard]] const std::map<u64, api::Section>& getSections() const;
 
         [[nodiscard]] u64 getSectionCount() const;
@@ -224,6 +277,8 @@ namespace pl::core {
         u64 getReadOffsetAndIncrement(u64 incrementSize);
 
         void setReadOffset(u64 offset);
+        void setStartAddress(u64 address);
+        u64 getStartAddress() const;
 
         void setReadOrderReversed(bool reversed) { this->m_readOrderReversed = reversed; }
 
@@ -277,26 +332,31 @@ namespace pl::core {
         void createParameterPack(const std::string &name, const std::vector<Token::Literal> &values);
 
         void createArrayVariable(const std::string &name, const ast::ASTNode *type, size_t entryCount, u64 section, bool constant = false);
-        std::shared_ptr<ptrn::Pattern> createVariable(const std::string &name, const ast::ASTNodeTypeDecl *type, const std::optional<Token::Literal> &value = std::nullopt, bool outVariable = false, bool reference = false, bool templateVariable = false, bool constant = false);
+        std::shared_ptr<ptrn::Pattern> createVariable(const std::string &name, const ast::ASTNodeTypeApplication *type, const std::optional<Token::Literal> &value = std::nullopt, bool outVariable = false, bool reference = false, bool templateVariable = false, bool constant = false);
         std::shared_ptr<ptrn::Pattern>& getVariableByName(const std::string &name);
         void setVariable(const std::string &name, const Token::Literal &value);
         void setVariable(std::shared_ptr<ptrn::Pattern> &pattern, const Token::Literal &value);
         void setVariableAddress(const std::string &variableName, u64 address, u64 section = 0);
         void changePatternSection(ptrn::Pattern *pattern, u64 section);
-        void changePatternType(std::shared_ptr<ptrn::Pattern> &pattern, std::shared_ptr<ptrn::Pattern> &&newPattern);
+        void changePatternType(std::shared_ptr<ptrn::Pattern> &pattern, std::shared_ptr<ptrn::Pattern> &&newPattern) const;
 
         void abort() {
             this->m_aborted = true;
+            for (auto &subRuntime : m_subRuntimes) {
+                subRuntime.abort();
+            }
         }
 
         void handleAbort() const {
-            if (this->m_aborted) [[unlikely]]
-                err::E0007.throwError("Evaluation aborted by user.");
+            if (this->m_aborted) [[unlikely]] {
+                if (std::uncaught_exceptions() == 0) {
+                    err::E0007.throwError("Evaluation aborted by user.");
+                }
+            }
         }
 
         [[nodiscard]] std::optional<Token::Literal> getEnvVariable(const std::string &name) const {
             if (this->m_envVariables.contains(name)) {
-                auto value = this->m_envVariables.at(name);
                 return this->m_envVariables.at(name);
             } else
                 return std::nullopt;
@@ -361,11 +421,12 @@ namespace pl::core {
 
         [[nodiscard]] Evaluator::UpdateHandler updateRuntime(const ast::ASTNode *node);
 
-        void addBreakpoint(u64 line);
-        void removeBreakpoint(u64 line);
+        void addBreakpoint(u32 line);
+        void removeBreakpoint(u32 line);
         void clearBreakpoints();
         void setBreakpointHitCallback(const std::function<void()> &callback);
-        const std::unordered_set<int>& getBreakpoints() const;
+        void setBreakpoints(const std::unordered_set<u32>& breakpoints);
+        const std::unordered_set<u32>& getBreakpoints() const;
         void pauseNextLine();
         std::optional<u32> getPauseLine() const;
 
@@ -381,30 +442,87 @@ namespace pl::core {
             return this->m_lastPatternAddress;
         }
 
+        std::set<std::string>& getStringPool() {
+            return this->m_stringPool;
+        }
+
+        const std::set<std::string>& getStringPool() const {
+            return this->m_stringPool;
+        }
+
+        bool isStringPoolEntryValid(const std::set<std::string>::const_iterator &it) const {
+            return it != this->m_stringPool.end();
+        }
+
+        PatternLanguage& createSubRuntime() {
+            return m_subRuntimes.emplace_back(this->m_patternLanguage->cloneRuntime());
+        }
+
+        PatternLanguage& getRuntime() {
+            return *m_patternLanguage;
+        }
+
+        void setPatternColorPalette(std::span<const u32> palette) {
+            m_patternColorPalette.clear();
+            m_patternColorPalette.reserve(palette.size());
+
+            std::ranges::copy(palette, std::back_inserter(m_patternColorPalette));
+            resetPatternColorPaletteIndex();
+        }
+
+        void resetPatternColorPaletteIndex() {
+            m_patternColorPaletteIndex = 0;
+        }
+
+        const std::set<ptrn::Pattern*>& getPatternsWithAttribute(const std::string &attribute) const {
+            if (const auto it = m_attributedPatterns.find(attribute); it != m_attributedPatterns.end()) {
+                return it->second;
+            } else {
+                static const std::set<ptrn::Pattern*> empty;
+                return empty;
+            }
+        }
+
     private:
-        void patternCreated(const ptrn::Pattern *pattern);
-        void patternDestroyed(const ptrn::Pattern *pattern);
+        void patternCreated(ptrn::Pattern *pattern);
+        void patternDestroyed(ptrn::Pattern *pattern);
 
         api::FunctionCallback handleDangerousFunctionCall(const std::string &functionName, const api::FunctionCallback &function);
 
+        void setRuntime(PatternLanguage *runtime) {
+            this->m_patternLanguage = runtime;
+        }
+
+
+        void addAttributedPattern(const std::string &attribute, ptrn::Pattern *pattern) {
+            m_attributedPatterns[attribute].insert(pattern);
+        }
+
+        void removeAttributedPattern(const std::string &attribute, ptrn::Pattern *pattern) {
+            if (const auto it = m_attributedPatterns.find(attribute); it != m_attributedPatterns.end())
+                it->second.erase(pattern);
+        }
+
     private:
+        PatternLanguage *m_patternLanguage = nullptr;
+        std::list<PatternLanguage> m_subRuntimes;
+
         u64 m_currOffset = 0x00;
         i8 m_currBitOffset = 0;
         bool m_readOrderReversed = false;
+        u64 m_startAddress = 0x00;
 
         bool m_evaluated = false;
         bool m_debugMode = false;
         LogConsole m_console;
 
-        u32 m_colorIndex = 0;
-
         std::endian m_defaultEndian = std::endian::native;
         u64 m_evalDepth = 0;
         u64 m_arrayLimit = 0;
         u64 m_patternLimit = 0;
         u64 m_loopLimit = 0;
 
-        u64 m_currPatternCount = 0;
+        std::atomic<u64> m_currPatternCount = 0;
 
         std::atomic<bool> m_aborted;
 
@@ -412,7 +530,13 @@ namespace pl::core {
         std::map<u64, api::Section> m_sections;
         u64 m_sectionId = 0;
 
+        std::vector<std::vector<u8>> m_heap;
+        std::map<u32, PatternLocalData> m_patternLocalStorage;
+
+        std::map<std::string, std::set<ptrn::Pattern*>> m_attributedPatterns;
         std::vector<std::unique_ptr<Scope>> m_scopes;
+        std::vector<std::shared_ptr<ptrn::Pattern>> m_patterns;
+
         std::unordered_map <std::string, api::Function> m_customFunctions;
         std::unordered_map <std::string, api::Function> m_builtinFunctions;
         std::vector<std::unique_ptr<ast::ASTNode>> m_customFunctionDefinitions;
@@ -424,17 +548,16 @@ namespace pl::core {
         std::map<std::string, std::shared_ptr<ptrn::Pattern>> m_outVariables;
         std::map<std::string, Token::Literal> m_outVariableValues;
         std::vector<std::vector<std::shared_ptr<ptrn::Pattern>>> m_templateParameters;
-
-        std::vector<std::vector<u8>> m_heap;
-        std::map<u32, PatternLocalData> m_patternLocalStorage;
+        std::vector<std::vector<std::shared_ptr<ast::ASTNode>>> m_typeTemplateParameters;
+        std::vector<std::unique_ptr<ast::ASTNode>> m_currentTemplateArguments;
 
         std::function<bool()> m_dangerousFunctionCalledCallback = []{ return false; };
         std::function<void()> m_breakpointHitCallback = []{ };
         std::atomic<DangerousFunctionPermission> m_allowDangerousFunctions = DangerousFunctionPermission::Ask;
         ControlFlowStatement m_currControlFlowStatement = ControlFlowStatement::None;
-        std::vector<std::unique_ptr<ast::ASTNode>> m_callStack;
+        std::vector<StackTrace> m_callStack;
 
-        std::vector<std::shared_ptr<ptrn::Pattern>> m_patterns;
+        std::set<std::string> m_stringPool;
 
         u64 m_dataBaseAddress = 0x00;
         u64 m_dataSize = 0x00;
@@ -442,28 +565,33 @@ namespace pl::core {
             err::E0011.throwError("No memory has been attached. Reading is disabled.");
         };
         std::function<void(u64, const u8*, size_t)> m_writerFunction = [](u64, const u8*, size_t){
-            err::E0011.throwError("No memory has been attached. Reading is disabled.");
+            err::E0011.throwError("No memory has been attached. Writing is disabled.");
         };
 
         bool m_mainSectionEditsAllowed = false;
 
         std::optional<u64> m_currArrayIndex;
 
-        std::unordered_set<int> m_breakpoints;
+        std::unordered_set<u32> m_breakpoints;
         std::optional<u32> m_lastPauseLine;
         bool m_shouldPauseNextLine = false;
 
         std::atomic<u64> m_lastReadAddress, m_lastWriteAddress, m_lastPatternAddress;
+        std::atomic<ptrn::Pattern*> m_lastPattern = nullptr;
+        std::vector<u32> m_sourceLineLength;
+
+        constexpr static std::array<u32, 9> DefaultPatternColorPalette = { 0x70B4771F, 0x700E7FFF, 0x702CA02C, 0x702827D6, 0x70BD6794, 0x704B568C, 0x70C277E3, 0x7022BDBC, 0x70CFBE17 };
+        std::vector<u32> m_patternColorPalette = { DefaultPatternColorPalette.begin(), DefaultPatternColorPalette.end() };
+        u32 m_patternColorPaletteIndex = 0;
 
         u32 getNextPatternColor() {
-            constexpr static std::array Palette = { 0x70B4771F, 0x700E7FFF, 0x702CA02C, 0x702827D6, 0x70BD6794, 0x704B568C, 0x70C277E3, 0x7022BDBC, 0x70CFBE17 };
+            const auto index = this->m_patternColorPaletteIndex;
+            this->m_patternColorPaletteIndex = (index + 1) % m_patternColorPalette.size();
 
-            auto index = this->m_colorIndex;
-            this->m_colorIndex = (this->m_colorIndex + 1) % Palette.size();
-
-            return Palette[index];
+            return m_patternColorPalette[index];
         }
 
+        friend class pl::PatternLanguage;
         friend class pl::ptrn::PatternCreationLimiter;
         friend class pl::ptrn::Pattern;
         friend struct UpdateHandler;
