@@ -9,14 +9,16 @@ namespace pl::ptrn {
     public:
         using Pattern::Pattern;
 
-        virtual void setParentBitfield(PatternBitfieldMember *parent) = 0;
-
-        [[nodiscard]] virtual const PatternBitfieldMember* getParentBitfield() const = 0;
-
         [[nodiscard]] const PatternBitfieldMember& getTopmostBitfield() const {
-            const pl::ptrn::PatternBitfieldMember* topBitfield = this;
-            while (auto parentBitfield = topBitfield->getParentBitfield())
+            const PatternBitfieldMember* topBitfield = this;
+            while (auto parent = topBitfield->getParent()) {
+                auto parentBitfield = dynamic_cast<const PatternBitfieldMember*>(parent);
+                if (parentBitfield == nullptr)
+                    break;
+
                 topBitfield = parentBitfield;
+            }
+
             return *topBitfield;
         }
 
@@ -52,16 +54,18 @@ namespace pl::ptrn {
     class PatternBitfieldField : public PatternBitfieldMember {
     public:
         PatternBitfieldField(core::Evaluator *evaluator, u64 offset, u8 bitOffset, u8 bitSize, u32 line, PatternBitfieldMember *parentBitfield = nullptr)
-                : PatternBitfieldMember(evaluator, offset, (bitOffset + bitSize + 7) / 8, line), m_parentBitfield(parentBitfield), m_bitOffset(bitOffset % 8), m_bitSize(bitSize) { }
+                : PatternBitfieldMember(evaluator, offset, (bitOffset + bitSize + 7) / 8, line), m_bitOffset(bitOffset % 8), m_bitSize(bitSize) {
+            if (parentBitfield != nullptr)
+                this->setParent(parentBitfield->reference());
+        }
 
         PatternBitfieldField(const PatternBitfieldField &other) : PatternBitfieldMember(other) {
             this->m_padding = other.m_padding;
             this->m_bitOffset = other.m_bitOffset;
             this->m_bitSize = other.m_bitSize;
-            this->m_parentBitfield = other.m_parentBitfield;
         }
 
-        [[nodiscard]] std::unique_ptr<Pattern> clone() const override {
+        [[nodiscard]] std::shared_ptr<Pattern> clone() const override {
             return std::unique_ptr<Pattern>(new PatternBitfieldField(*this));
         }
 
@@ -73,14 +77,6 @@ namespace pl::ptrn {
             return transformValue(this->readValue());
         }
 
-        void setParentBitfield(PatternBitfieldMember *parentBitfield) override {
-            this->m_parentBitfield = parentBitfield;
-        }
-
-        const PatternBitfieldMember* getParentBitfield() const override {
-            return this->m_parentBitfield;
-        }
-
         [[nodiscard]] std::string getFormattedName() const override {
             return this->m_bitSize == 1 ? "bit" : "bits";
         }
@@ -115,7 +111,7 @@ namespace pl::ptrn {
             return Pattern::callUserFormatFunc(literal).value_or(fmt::format("{}", value));
         }
 
-        [[nodiscard]] std::string toString() const override {
+        [[nodiscard]] std::string toString() override {
             auto value = this->readValue();
             return Pattern::callUserFormatFunc(value, true).value_or(fmt::format("{}", value));
         }
@@ -151,13 +147,11 @@ namespace pl::ptrn {
                 this->getEvaluator()->writeBits(this->getOffset(), this->getBitOffset(), this->getBitSize(), this->getSection(), this->getEndian(), writeValue);
 
                 this->clearFormatCache();
-                this->m_parentBitfield->clearFormatCache();
+                this->getParent()->clearFormatCache();
             }
         }
 
     private:
-        PatternBitfieldMember *m_parentBitfield = nullptr;
-
         u8 m_bitOffset;
         u8 m_bitSize;
 
@@ -168,7 +162,7 @@ namespace pl::ptrn {
     public:
         using PatternBitfieldField::PatternBitfieldField;
 
-        [[nodiscard]] std::unique_ptr<Pattern> clone() const override {
+        [[nodiscard]] std::shared_ptr<Pattern> clone() const override {
             return std::unique_ptr<Pattern>(new PatternBitfieldFieldSigned(*this));
         }
 
@@ -178,11 +172,11 @@ namespace pl::ptrn {
 
         std::string formatDisplayValue() override {
             auto rawValue = this->readValue();
-            auto value = hlp::signExtend(this->getBitSize(), rawValue);
+            auto value = hlp::signExtend(this->getBitSize(), i128(rawValue));
             return Pattern::callUserFormatFunc(value).value_or(fmt::format("{}", value));
         }
 
-        [[nodiscard]] std::string toString() const override {
+        [[nodiscard]] std::string toString() override {
             auto result = fmt::format("{}", this->getValue().toSigned());
             return Pattern::callUserFormatFunc(this->getValue(), true).value_or(result);
         }
@@ -192,7 +186,7 @@ namespace pl::ptrn {
     public:
         using PatternBitfieldField::PatternBitfieldField;
 
-        [[nodiscard]] std::unique_ptr<Pattern> clone() const override {
+        [[nodiscard]] std::shared_ptr<Pattern> clone() const override {
             return std::unique_ptr<Pattern>(new PatternBitfieldFieldBoolean(*this));
         }
 
@@ -205,14 +199,16 @@ namespace pl::ptrn {
         }
 
         std::string formatDisplayValue() override {
-            switch (this->getValue().toUnsigned()) {
-                case 0: return "false";
-                case 1: return "true";
-                default: return "true*";
-            }
+            auto value = this->getValue().toUnsigned();
+            if (value == 0)
+                return "false";
+            else if (value == 1)
+                return "true";
+            else
+                return "true*";
         }
 
-        [[nodiscard]] std::string toString() const override {
+        [[nodiscard]] std::string toString() override {
             auto value = this->getValue();
             return Pattern::callUserFormatFunc(value, true).value_or(fmt::format("{}", value.toBoolean() ? "true" : "false"));
         }
@@ -226,10 +222,6 @@ namespace pl::ptrn {
             return "enum " + Pattern::getTypeName();
         }
 
-        [[nodiscard]] std::string getTypeName() const override {
-            return Pattern::getTypeName();
-        }
-
         void setEnumValues(const std::map<std::string, PatternEnum::EnumValue> &enumValues) {
             this->m_enumValues = enumValues;
         }
@@ -249,7 +241,7 @@ namespace pl::ptrn {
             return true;
         }
 
-        [[nodiscard]] std::unique_ptr<Pattern> clone() const override {
+        [[nodiscard]] std::shared_ptr<Pattern> clone() const override {
             return std::unique_ptr<Pattern>(new PatternBitfieldFieldEnum(*this));
         }
 
@@ -259,7 +251,7 @@ namespace pl::ptrn {
             return Pattern::callUserFormatFunc(value).value_or(fmt::format("{}", enumName));
         }
 
-        [[nodiscard]] std::string toString() const override {
+        [[nodiscard]] std::string toString() override {
             auto enumName = PatternEnum::getEnumName(this->getTypeName(), this->readValue(), this->getEnumValues());
             return Pattern::callUserFormatFunc(this->getValue(), true).value_or(enumName);
         }
@@ -273,7 +265,7 @@ namespace pl::ptrn {
                                  public IIndexable {
     public:
         PatternBitfieldArray(core::Evaluator *evaluator, u64 offset, u8 firstBitOffset, u128 totalBitSize, u32 line)
-                : PatternBitfieldMember(evaluator, offset, (totalBitSize + 7) / 8, line), m_firstBitOffset(firstBitOffset), m_totalBitSize(totalBitSize) { }
+                : PatternBitfieldMember(evaluator, offset, size_t((totalBitSize + 7) / 8), line), m_firstBitOffset(firstBitOffset), m_totalBitSize(totalBitSize) { }
 
         PatternBitfieldArray(const PatternBitfieldArray &other) : PatternBitfieldMember(other) {
             std::vector<std::shared_ptr<Pattern>> entries;
@@ -286,16 +278,11 @@ namespace pl::ptrn {
             this->m_totalBitSize = other.m_totalBitSize;
         }
 
-        [[nodiscard]] std::unique_ptr<Pattern> clone() const override {
-            return std::unique_ptr<Pattern>(new PatternBitfieldArray(*this));
-        }
-
-        void setParentBitfield(PatternBitfieldMember *parentBitfield) override {
-            this->m_parentBitfield = parentBitfield;
-        }
-
-        const PatternBitfieldMember* getParentBitfield() const override {
-            return this->m_parentBitfield;
+        [[nodiscard]] std::shared_ptr<Pattern> clone() const override {
+            auto other =  std::make_shared<PatternBitfieldArray>(*this);
+            for (const auto &entry : other->m_entries)
+                entry->setParent(other->reference());
+            return other;
         }
 
         [[nodiscard]] u8 getBitOffset() const override {
@@ -308,11 +295,11 @@ namespace pl::ptrn {
 
         void setBitSize(u128 bitSize) {
             this->m_totalBitSize = bitSize;
-            this->setSize((bitSize + 7) / 8);
+            this->setSize(size_t((bitSize + 7) / 8));
         }
 
         [[nodiscard]] u64 getBitSize() const override {
-            return this->m_totalBitSize;
+            return u64(this->m_totalBitSize);
         }
 
         [[nodiscard]] bool isReversed() const {
@@ -397,18 +384,14 @@ namespace pl::ptrn {
 
         void setOffset(u64 offset) override {
             for (auto &entry : this->m_entries) {
-                if (entry->getSection() == this->getSection()) {
-                    if (entry->getSection() != ptrn::Pattern::PatternLocalSectionId)
-                        entry->setOffset(entry->getOffset() - this->getOffset() + offset);
-                    else
-                        entry->setOffset(offset);
-                }
+                if (entry->getSection() == this->getSection() && entry->getSection() != ptrn::Pattern::PatternLocalSectionId)
+                    entry->setOffset(entry->getOffset() - this->getOffset() + offset);
             }
 
             PatternBitfieldMember::setOffset(offset);
         }
 
-        void forEachEntry(u64 start, u64 end, const std::function<void(u64, Pattern*)>& fn) override {
+        void forEachEntry(u64 start, u64 end, const std::function<void(u64, const std::shared_ptr<Pattern>&)>& fn) override {
             auto evaluator = this->getEvaluator();
             auto startArrayIndex = evaluator->getCurrentArrayIndex();
 
@@ -424,7 +407,7 @@ namespace pl::ptrn {
 
                 auto &entry = this->m_entries[i];
                 if (!entry->isPatternLocal() || entry->hasAttribute("export"))
-                    fn(i, entry.get());
+                    fn(i, entry);
             }
         }
 
@@ -435,8 +418,6 @@ namespace pl::ptrn {
                 if (!entry->hasOverriddenColor())
                     entry->setBaseColor(this->getColor());
 
-                entry->setParent(this);
-
                 this->m_sortedEntries.push_back(entry.get());
             }
 
@@ -444,7 +425,7 @@ namespace pl::ptrn {
                 this->setBaseColor(this->m_entries.front()->getColor());
         }
 
-        [[nodiscard]] std::string toString() const override {
+        [[nodiscard]] std::string toString() override {
             std::string result;
 
             result += "[ ";
@@ -468,7 +449,7 @@ namespace pl::ptrn {
 
             result += " ]";
 
-            return Pattern::callUserFormatFunc(this->clone(), true).value_or(result);
+            return Pattern::callUserFormatFunc(this->reference(), true).value_or(result);
         }
 
         [[nodiscard]] bool operator==(const Pattern &other) const override {
@@ -508,7 +489,7 @@ namespace pl::ptrn {
         }
 
         std::string formatDisplayValue() override {
-            return Pattern::callUserFormatFunc(this->clone()).value_or("[ ... ]");
+            return Pattern::callUserFormatFunc(this->reference()).value_or("[ ... ]");
         }
 
         void sort(const std::function<bool (const Pattern *, const Pattern *)> &comparator) override {
@@ -536,7 +517,7 @@ namespace pl::ptrn {
         }
 
         void clearFormatCache() override {
-            this->forEachEntry(0, this->getEntryCount(), [&](u64, pl::ptrn::Pattern *entry) {
+            this->forEachEntry(0, this->getEntryCount(), [&](u64, const auto &entry) {
                 entry->clearFormatCache();
             });
 
@@ -548,7 +529,6 @@ namespace pl::ptrn {
         std::vector<Pattern *> m_sortedEntries;
         u8 m_firstBitOffset = 0;
         u128 m_totalBitSize = 0;
-        PatternBitfieldMember *m_parentBitfield = nullptr;
         bool m_reversed = false;
     };
 
@@ -557,29 +537,20 @@ namespace pl::ptrn {
                             public IIterable {
     public:
         PatternBitfield(core::Evaluator *evaluator, u64 offset, u8 firstBitOffset, u128 totalBitSize, u32 line)
-                : PatternBitfieldMember(evaluator, offset, (totalBitSize + 7) / 8, line), m_firstBitOffset(firstBitOffset), m_totalBitSize(totalBitSize) { }
+                : PatternBitfieldMember(evaluator, offset, size_t((totalBitSize + 7) / 8), line), m_firstBitOffset(firstBitOffset), m_totalBitSize(totalBitSize) { }
 
         PatternBitfield(const PatternBitfield &other) : PatternBitfieldMember(other) {
-            for (auto &field : other.m_fields)
+            for (const auto &field : other.m_fields)
                 this->m_fields.push_back(field->clone());
 
-            this->m_parentBitfield = other.m_parentBitfield;
             this->m_firstBitOffset = other.m_firstBitOffset;
             this->m_totalBitSize = other.m_totalBitSize;
         }
 
-        [[nodiscard]] std::unique_ptr<Pattern> clone() const override {
+        [[nodiscard]] std::shared_ptr<Pattern> clone() const override {
             return std::unique_ptr<Pattern>(new PatternBitfield(*this));
         }
 
-        void setParentBitfield(PatternBitfieldMember *parentBitfield) override {
-            this->m_parentBitfield = parentBitfield;
-        }
-
-        const PatternBitfieldMember* getParentBitfield() const override {
-            return this->m_parentBitfield;
-        }
-
         [[nodiscard]] u8 getBitOffset() const override {
             return m_firstBitOffset;
         }
@@ -589,8 +560,8 @@ namespace pl::ptrn {
         }
 
         void setBitSize(u128 bitSize) {
-            this->m_totalBitSize = bitSize;
-            this->setSize((bitSize + 7) / 8);
+            this->m_totalBitSize = u64(bitSize);
+            this->setSize(size_t((bitSize + 7) / 8));
         }
 
         [[nodiscard]] u64 getBitSize() const override {
@@ -666,7 +637,7 @@ namespace pl::ptrn {
                 this->setBaseColor(this->m_fields.front()->getColor());
 
             for (const auto &field : this->m_fields) {
-                field->setParent(this);
+                field->setParent(this->reference());
                 this->m_sortedFields.push_back(field.get());
             }
         }
@@ -696,7 +667,7 @@ namespace pl::ptrn {
             v.visit(*this);
         }
 
-        [[nodiscard]] std::string toString() const override {
+        [[nodiscard]] std::string toString() override {
             std::string result = this->getFormattedName();
 
             result += " { ";
@@ -714,7 +685,7 @@ namespace pl::ptrn {
 
             result += " }";
 
-            return Pattern::callUserFormatFunc(this->clone(), true).value_or(result);
+            return Pattern::callUserFormatFunc(this->reference(), true).value_or(result);
         }
 
         std::string formatDisplayValue() override {
@@ -748,9 +719,9 @@ namespace pl::ptrn {
             }
 
             if (valueString.size() > 64)
-                return Pattern::callUserFormatFunc(this->clone()).value_or(fmt::format("{{ ... }}", valueString));
+                return Pattern::callUserFormatFunc(this->reference()).value_or(fmt::format("{{ ... }}", valueString));
             else
-                return Pattern::callUserFormatFunc(this->clone()).value_or(fmt::format("{{ {} }}", valueString));
+                return Pattern::callUserFormatFunc(this->reference()).value_or(fmt::format("{{ {} }}", valueString));
         }
 
         void setEndian(std::endian endian) override {
@@ -780,25 +751,21 @@ namespace pl::ptrn {
 
         void setOffset(u64 offset) override {
             for (auto &field : this->m_fields) {
-                if (field->getSection() == this->getSection()) {
-                    if (field->getSection() != ptrn::Pattern::PatternLocalSectionId)
-                        field->setOffset(field->getOffset() - this->getOffset() + offset);
-                    else
-                        field->setOffset(offset);
-                }
+                if (field->getSection() == this->getSection() && field->getSection() != ptrn::Pattern::PatternLocalSectionId)
+                    field->setOffset(field->getOffset() - this->getOffset() + offset);
             }
 
             PatternBitfieldMember::setOffset(offset);
         }
 
-        void forEachEntry(u64 start, u64 end, const std::function<void (u64, Pattern *)> &fn) override {
+        void forEachEntry(u64 start, u64 end, const std::function<void (u64, const std::shared_ptr<Pattern>&)> &fn) override {
             if (this->isSealed())
                 return;
 
             for (auto i = start; i < end; i++) {
                 auto &pattern = this->m_fields[i];
                 if (!pattern->isPatternLocal() || pattern->hasAttribute("export"))
-                    fn(i, pattern.get());
+                    fn(i, pattern);
             }
         }
 
@@ -827,7 +794,7 @@ namespace pl::ptrn {
         }
 
         void clearFormatCache() override {
-            this->forEachEntry(0, this->getEntryCount(), [&](u64, pl::ptrn::Pattern *entry) {
+            this->forEachEntry(0, this->getEntryCount(), [&](u64, const auto &entry) {
                 entry->clearFormatCache();
             });
 
@@ -838,7 +805,6 @@ namespace pl::ptrn {
         std::vector<std::shared_ptr<Pattern>> m_fields;
         std::vector<Pattern *> m_sortedFields;
 
-        PatternBitfieldMember *m_parentBitfield = nullptr;
         u8 m_firstBitOffset = 0;
         u64 m_totalBitSize = 0;
         bool m_reversed = false;
