@@ -69,7 +69,7 @@ namespace pl::core::ast {
                         auto offsetNode = (*arraySegment)->evaluate(evaluator);
                         auto offsetLiteral = dynamic_cast<ASTNodeLiteral*>(offsetNode.get());
                         if (offsetLiteral != nullptr) {
-                            auto offset = offsetLiteral->getValue().toUnsigned();
+                            auto offset = u64(offsetLiteral->getValue().toUnsigned());
 
                             u8 byte = 0x00;
                             evaluator->readData(offset, &byte, 1, ptrn::Pattern::MainSectionId);
@@ -82,9 +82,10 @@ namespace pl::core::ast {
 
         std::shared_ptr<ptrn::Pattern> pattern;
         {
-            auto referencedPattern = std::move(this->createPatterns(evaluator).front());
+            std::vector<std::shared_ptr<ptrn::Pattern>> referencedPatterns;
+            this->createPatterns(evaluator, referencedPatterns);
 
-            pattern = referencedPattern;
+            pattern = std::move(referencedPatterns.front());
         }
 
         Token::Literal literal;
@@ -128,6 +129,8 @@ namespace pl::core::ast {
             literal = bool(bitfieldFieldPatternBoolean->readValue());
         } else if (auto bitfieldFieldPatternSigned = dynamic_cast<ptrn::PatternBitfieldFieldSigned *>(pattern.get()); bitfieldFieldPatternSigned != nullptr) {
             literal = hlp::signExtend(bitfieldFieldPatternSigned->getBitSize(), i128(bitfieldFieldPatternSigned->readValue()));
+        } else if (auto bitfieldFieldPatternEnum = dynamic_cast<ptrn::PatternBitfieldFieldEnum *>(pattern.get()); bitfieldFieldPatternEnum != nullptr) {
+            literal = pattern;
         } else if (auto bitfieldFieldPattern = dynamic_cast<ptrn::PatternBitfieldField *>(pattern.get()); bitfieldFieldPattern != nullptr) {
             literal = bitfieldFieldPattern->readValue();
         } else {
@@ -147,29 +150,13 @@ namespace pl::core::ast {
         return std::unique_ptr<ASTNode>(new ASTNodeLiteral(std::move(literal)));
     }
 
-    [[nodiscard]] std::vector<std::shared_ptr<ptrn::Pattern>> ASTNodeRValue::createPatterns(Evaluator *evaluator) const {
+    void ASTNodeRValue::createPatterns(Evaluator *evaluator, std::vector<std::shared_ptr<ptrn::Pattern>> &resultPatterns) const {
         [[maybe_unused]] auto context = evaluator->updateRuntime(this);
 
-        std::vector<std::shared_ptr<ptrn::Pattern>> searchScope;
         std::shared_ptr<ptrn::Pattern> currPattern;
         i32 scopeIndex = 0;
         bool iterable = true;
 
-        if (!evaluator->isGlobalScope()) {
-            const auto &globalScope = evaluator->getGlobalScope().scope;
-            std::copy(globalScope->begin(), globalScope->end(), std::back_inserter(searchScope));
-        }
-
-        {
-            const auto &currScope = evaluator->getScope(0);
-            std::copy(currScope.scope->begin(), currScope.scope->end(), std::back_inserter(searchScope));
-        }
-
-        {
-            const auto &templateParameters = evaluator->getTemplateParameters();
-            std::copy(templateParameters.begin(), templateParameters.end(), std::back_inserter(searchScope));
-        }
-
         for (const auto &part : this->getPath()) {
 
             if (!iterable)
@@ -177,7 +164,7 @@ namespace pl::core::ast {
 
             if (part.index() == 0) {
                 // Variable access
-                auto name = std::get<std::string>(part);
+                const auto &name = std::get<std::string>(part);
 
                 if (name == "parent") {
                     do {
@@ -187,7 +174,6 @@ namespace pl::core::ast {
                             err::E0003.throwError("Cannot access parent of global scope.", {}, this->getLocation());
                     } while (evaluator->getScope(scopeIndex).parent == nullptr);
 
-                    searchScope     = *evaluator->getScope(scopeIndex).scope;
                     auto currParent = evaluator->getScope(scopeIndex).parent;
 
                     if (currParent == nullptr) {
@@ -198,8 +184,6 @@ namespace pl::core::ast {
 
                     continue;
                 } else if (name == "this") {
-                    searchScope = *evaluator->getScope(scopeIndex).scope;
-
                     auto currParent = evaluator->getScope(0).parent;
 
                     if (currParent == nullptr)
@@ -208,13 +192,33 @@ namespace pl::core::ast {
                     currPattern = currParent;
                     continue;
                 } else {
-                    bool found = false;
-                    for (auto iter = searchScope.crbegin(); iter != searchScope.crend(); ++iter) {
-                        if ((*iter)->getVariableName() == name) {
-                            currPattern = *iter;
-                            found       = true;
-                            break;
-                        }
+                    using namespace std::views;
+                    using std::ranges::find;
+                    std::shared_ptr<ptrn::Pattern> pattern;
+
+                    if (currPattern == nullptr) {
+                        auto currScope = *evaluator->getScope(0).scope | all;
+                        auto templateParameters = evaluator->getTemplateParameters() | all;
+                        auto globalScope = *evaluator->getGlobalScope().scope | all;
+                        auto allScopes = {globalScope, templateParameters, currScope};
+                        auto searchScopes = allScopes | drop(evaluator->isGlobalScope() ? 1 : 0);
+                        auto view = searchScopes | join | reverse;
+                        if (auto result = find(view, name, &ptrn::Pattern::getVariableName); result != view.end())
+                            pattern = *result;
+                    } else if (auto currParent = evaluator->getScope(scopeIndex).parent; currParent == currPattern) {
+                        auto view = *evaluator->getScope(scopeIndex).scope | reverse;
+                        if (auto result = find(view, name, &ptrn::Pattern::getVariableName); result != view.end())
+                            pattern = *result;
+                    } else if (auto indexablePattern = dynamic_cast<ptrn::IIndexable *>(currPattern.get()); indexablePattern != nullptr) {
+                        auto iota_view = iota((size_t)0, indexablePattern->getEntryCount());
+                        auto view = iota_view | transform(std::bind_front(&ptrn::IIndexable::getEntry, indexablePattern)) | reverse;
+                        if (auto result = find(view, name, &ptrn::Pattern::getVariableName); result != view.end())
+                            pattern = *result;
+                    } else if (auto iterablePattern = dynamic_cast<ptrn::IIterable *>(currPattern.get()); iterablePattern != nullptr) {
+                        auto scope = iterablePattern->getEntries();
+                        auto view =  scope | reverse;
+                        if (auto result = find(view, name, &ptrn::Pattern::getVariableName); result != view.end())
+                            pattern = *result;
                     }
 
                     if (name == "$")
@@ -222,8 +226,10 @@ namespace pl::core::ast {
                     else if (name == "null")
                         err::E0003.throwError("Invalid use of 'null' keyword in rvalue.", {}, this->getLocation());
 
-                    if (!found)
+                    if (pattern == nullptr)
                         err::E0003.throwError(fmt::format("No variable named '{}' found.", name), {}, this->getLocation());
+
+                    currPattern = std::move(pattern);
                 }
             } else {
                 // Array indexing
@@ -239,8 +245,8 @@ namespace pl::core::ast {
                                        auto pattern = currPattern.get();
                                        if (auto indexablePattern = dynamic_cast<ptrn::IIndexable *>(pattern); indexablePattern != nullptr) {
                                            if (size_t(index) >= indexablePattern->getEntryCount())
-                                               core::err::E0006.throwError("Index out of bounds.", fmt::format("Tried to access index {} in array of size {}.", index, indexablePattern->getEntryCount()), this->getLocation());
-                                           currPattern = indexablePattern->getEntry(index);
+                                               core::err::E0006.throwError("Index out of bounds.", fmt::format("Tried to access index {} in array of size {}.", size_t(index), indexablePattern->getEntryCount()), this->getLocation());
+                                           currPattern = indexablePattern->getEntry(size_t(index));
                                        } else {
                                            err::E0006.throwError(fmt::format("Cannot access non-array type '{}'.", pattern->getTypeName()), {}, this->getLocation());
                                        }
@@ -259,9 +265,7 @@ namespace pl::core::ast {
 
             auto indexPattern = currPattern.get();
 
-            if (auto iterablePattern = dynamic_cast<ptrn::IIterable *>(indexPattern); iterablePattern != nullptr)
-                searchScope = iterablePattern->getEntries();
-            else
+            if (auto iterablePattern = dynamic_cast<ptrn::IIterable *>(indexPattern); iterablePattern == nullptr)
                 iterable = false;
 
         }
@@ -269,7 +273,7 @@ namespace pl::core::ast {
         if (currPattern == nullptr)
             err::E0003.throwError("Cannot reference global scope.", {}, this->getLocation());
         else
-            return hlp::moveToVector(std::move(currPattern));
+            resultPatterns = hlp::moveToVector(std::move(currPattern));
     }
 
 }
\ No newline at end of file
