@@ -42,7 +42,16 @@ namespace pl::core::ast {
                 evaluator->popSectionId();
             };
 
-            auto patterns = this->m_expression->createPatterns(evaluator);
+            std::vector<std::shared_ptr<ptrn::Pattern>> patterns;
+            if (this->getOperator() == Token::Operator::TypeNameOf) {
+                if (auto typeApp = dynamic_cast<ASTNodeTypeApplication*>(this->m_expression.get()); typeApp != nullptr) {
+                    auto evaluatedType = typeApp->evaluate(evaluator);
+                    result = dynamic_cast<ASTNodeTypeApplication*>(evaluatedType.get())->getTypeName();
+                    return std::unique_ptr<ASTNode>(new ASTNodeLiteral(result));
+                }
+            }
+
+            this->m_expression->createPatterns(evaluator, patterns);
             if (patterns.empty())
                 err::E0005.throwError("'auto' can only be used with parameters.", { }, this->getLocation());
 
@@ -55,24 +64,9 @@ namespace pl::core::ast {
                 case Token::Operator::SizeOf:
                     result = u128(pattern->getSize());
                     break;
-                case Token::Operator::TypeNameOf: {
-                    if (auto typeDecl = dynamic_cast<ASTNodeTypeDecl*>(this->m_expression.get()); typeDecl != nullptr) {
-                        ASTNodeTypeDecl *node;
-                        while (true) {
-                            node = dynamic_cast<ASTNodeTypeDecl*>(typeDecl->getType().get());
-                            if (node != nullptr)
-                                typeDecl = node;
-                            else
-                                break;
-                        }
-
-                        result = typeDecl->getName();
-                    } else {
-                        result = pattern->getTypeName();
-                    }
-
+                case Token::Operator::TypeNameOf:
+                    result = pattern->getTypeName();
                     break;
-                }
                 default:
                     err::E0001.throwError("Invalid type operation.", {}, this->getLocation());
             }
