@@ -23,6 +23,7 @@
 #include <pl/patterns/pattern_string.hpp>
 #include <pl/patterns/pattern_array_dynamic.hpp>
 #include <pl/patterns/pattern_padding.hpp>
+#include <pl/patterns/pattern_error.hpp>
 
 #include <exception>
 #include <utility>
@@ -69,6 +70,15 @@ namespace pl::core {
         this->m_currBitOffset = 0;
     }
 
+    void Evaluator::setStartAddress(u64 address) {
+        this->m_startAddress = address;
+    }
+
+    u64 Evaluator::getStartAddress() const {
+        return this->m_startAddress;
+    }
+
+
     [[nodiscard]] ByteAndBitOffset Evaluator::getBitwiseReadOffsetAndIncrement(i128 bitSize) {
         ByteAndBitOffset readOffsets = {};
 
@@ -77,8 +87,8 @@ namespace pl::core {
         else
             readOffsets = getBitwiseReadOffset();
 
-        this->m_currOffset += bitSize >> 3;
-        this->m_currBitOffset += bitSize & 0x7;
+        this->m_currOffset += u64(bitSize >> 3);
+        this->m_currBitOffset += i8(bitSize & 0x7);
 
         this->m_currOffset += this->m_currBitOffset >> 3;
         this->m_currBitOffset &= 0x7;
@@ -106,7 +116,7 @@ namespace pl::core {
 
         size_t readSize = (bitOffset + bitSize + 7) / 8;
         readSize = std::min(readSize, sizeof(value));
-        this->readData(byteOffset, &value, readSize, section);
+        this->readData(u64(byteOffset), &value, readSize, section);
         value = hlp::changeEndianess(value, sizeof(value), endianness);
 
         size_t offset = endianness == std::endian::little ? bitOffset : (sizeof(value) * 8) - bitOffset - bitSize;
@@ -125,14 +135,14 @@ namespace pl::core {
         value = (value & mask) << offset;
 
         u128 oldValue = 0;
-        this->readData(byteOffset, &oldValue, writeSize, section);
+        this->readData(u64(byteOffset), &oldValue, writeSize, section);
         oldValue = hlp::changeEndianess(oldValue, sizeof(oldValue), endianness);
 
         oldValue &= ~(mask << offset);
         oldValue |= value;
 
         oldValue = hlp::changeEndianess(oldValue, sizeof(oldValue), endianness);
-        this->writeData(byteOffset, &oldValue, writeSize, section);
+        this->writeData(u64(byteOffset), &oldValue, writeSize, section);
     }
 
 
@@ -172,6 +182,9 @@ namespace pl::core {
     }
 
     [[nodiscard]] std::optional<api::Function> Evaluator::findFunction(const std::string &name) const {
+        if (name.empty())
+            return std::nullopt;
+
         const auto &customFunctions     = this->getCustomFunctions();
         const auto &builtinFunctions    = this->getBuiltinFunctions();
 
@@ -203,7 +216,9 @@ namespace pl::core {
         }
 
         auto startOffset = this->getBitwiseReadOffset();
-        auto typePatterns = type->createPatterns(this);
+
+        std::vector<std::shared_ptr<ptrn::Pattern>> typePatterns;
+        type->createPatterns(this, typePatterns);
         auto typePattern = std::move(typePatterns.front());
 
         typePattern->setConstant(constant);
@@ -265,79 +280,7 @@ namespace pl::core {
         variables.push_back(std::unique_ptr<ptrn::Pattern>(pattern));
     }
 
-    std::optional<std::string> Evaluator::findTypeName(const ast::ASTNodeTypeDecl *type) {
-        const ast::ASTNodeTypeDecl *typeDecl = type;
-        while (true) {
-            if (auto name = typeDecl->getName(); !name.empty()) {
-                if (const auto &templateParams = typeDecl->getTemplateParameters(); templateParams.empty()) {
-                    return name;
-                } else {
-                    std::string templateTypeString;
-                    for (const auto &templateParameter : templateParams) {
-                        if (auto lvalue = dynamic_cast<ast::ASTNodeLValueAssignment *>(templateParameter.get())) {
-                            if (!lvalue->getRValue())
-                                err::E0003.throwError(fmt::format("No value set for non-type template parameter {}. This is a bug.", lvalue->getLValueName()), {}, type->getLocation());
-                            auto valueNode = lvalue->getRValue()->evaluate(this);
-                            if (auto literal = dynamic_cast<ast::ASTNodeLiteral*>(valueNode.get()); literal != nullptr) {
-                                const auto &value = literal->getValue();
-
-                                if (value.isString()) {
-                                    auto string = value.toString();
-                                    if (string.size() > 32)
-                                        string = "...";
-                                    templateTypeString += fmt::format("\"{}\", ", hlp::encodeByteString({ string.begin(), string.end() }));
-                                } else if (value.isPattern()) {
-                                    templateTypeString += fmt::format("{}{{ }}, ", value.toPattern()->getTypeName());
-                                } else {
-                                    templateTypeString += fmt::format("{}, ", value.toString(true));
-                                }
-                            } else {
-                                err::E0003.throwError(fmt::format("Template parameter {} is not a literal. This is a bug.", lvalue->getLValueName()), {}, type->getLocation());
-                            }
-                        } else if (const auto *typeNode = dynamic_cast<ast::ASTNodeTypeDecl*>(templateParameter.get())) {
-                            const auto *node = typeNode->getType().get();
-                            while (node != nullptr) {
-                                if (const auto *innerNode = dynamic_cast<const ast::ASTNodeTypeDecl*>(node)) {
-                                    if (const auto &innerNodeName = innerNode->getName(); !innerNodeName.empty()) {
-                                        templateTypeString += fmt::format("{}, ", innerNodeName);
-                                        break;
-                                    }
-                                    node = innerNode->getType().get();
-                                }
-                                if (const auto *innerNode = dynamic_cast<const ast::ASTNodeBuiltinType*>(node)) {
-                                    templateTypeString += fmt::format("{}, ", Token::getTypeName(innerNode->getType()));
-
-                                    break;
-                                }
-                            }
-                        }
-                    }
-
-                    templateTypeString = templateTypeString.substr(0, templateTypeString.size() - 2);
-
-                    return fmt::format("{}<{}>", name, templateTypeString);
-                }
-            }
-            else if (auto innerType = dynamic_cast<ast::ASTNodeTypeDecl*>(typeDecl->getType().get()); innerType != nullptr)
-                typeDecl = innerType;
-            else
-                return std::nullopt;
-        }
-    }
-
-    static ast::ASTNodeBuiltinType* getBuiltinType(const ast::ASTNodeTypeDecl *type) {
-        const ast::ASTNodeTypeDecl *typeDecl = type;
-        while (true) {
-            if (auto innerType = dynamic_cast<ast::ASTNodeTypeDecl*>(typeDecl->getType().get()); innerType != nullptr)
-                typeDecl = innerType;
-            else if (auto builtinType = dynamic_cast<ast::ASTNodeBuiltinType*>(typeDecl->getType().get()); builtinType != nullptr)
-                return builtinType;
-            else
-                return nullptr;
-        }
-    }
-
-    std::shared_ptr<ptrn::Pattern> Evaluator::createVariable(const std::string &name, const ast::ASTNodeTypeDecl *type, const std::optional<Token::Literal> &value, bool outVariable, bool reference, bool templateVariable, bool constant) {
+    std::shared_ptr<ptrn::Pattern> Evaluator::createVariable(const std::string &name, const ast::ASTNodeTypeApplication *type, const std::optional<Token::Literal> &value, bool outVariable, bool reference, bool templateVariable, bool constant) {
         auto startPos = this->getBitwiseReadOffset();
         ON_SCOPE_EXIT { this->setBitwiseReadOffset(startPos); };
 
@@ -378,7 +321,8 @@ namespace pl::core {
 
         this->setBitwiseReadOffset(startOffset);
 
-        if (auto builtinType = getBuiltinType(type); builtinType != nullptr && builtinType->getType() == Token::ValueType::Auto) {
+        auto typeDefinition = type->getTypeDefinition(this);
+        if (auto builtinType = dynamic_cast<const ast::ASTNodeBuiltinType*>(typeDefinition); builtinType != nullptr && builtinType->getType() == Token::ValueType::Auto) {
             // Handle auto variables
             if (!value.has_value())
                 pattern = std::make_shared<ptrn::PatternPadding>(this, 0, 0, 0);
@@ -403,15 +347,14 @@ namespace pl::core {
             else
                 err::E0003.throwError("Cannot determine type of 'auto' variable.", "Try initializing it directly with a literal.", type->getLocation());
         } else {
-            if (builtinType != nullptr)
-                pattern = builtinType->createPatterns(this).front();
+            if (builtinType != nullptr) {
+                std::vector<std::shared_ptr<ptrn::Pattern>> patterns;
+                type->createPatterns(this, patterns);
+                pattern = std::move(patterns.front());
+            }
             else {
                 pattern = std::make_shared<ptrn::PatternPadding>(this, 0, 0, 0);
-
-                if (auto typeName = findTypeName(type); typeName.has_value())
-                    pattern->setTypeName(typeName.value());
-                else
-                    err::E0003.throwError("Cannot determine type.", "", type->getLocation());
+                pattern->setTypeName(type->getTypeName());
             }
         }
 
@@ -455,7 +398,7 @@ namespace pl::core {
         std::memcpy(&result, &value, std::min(sizeof(result), bytes));
 
         if (std::signed_integral<T>)
-            result = hlp::signExtend(bytes * 8, result);
+            result = hlp::signExtend(bytes * 8, i128(result));
 
         return result;
     }
@@ -823,7 +766,6 @@ namespace pl::core {
         if (this->isDebugModeEnabled())
             this->getConsole().log(LogConsole::Level::Debug, fmt::format("Exiting scope #{}. Parent: '{}', Heap Size: {}.", this->m_scopes.size(), currScope.parent == nullptr ? "None" : currScope.parent->getVariableName(), heap.size()));
 
-
         this->m_scopes.pop_back();
     }
 
@@ -920,6 +862,9 @@ namespace pl::core {
     }
 
     void Evaluator::popSectionId() {
+        if (this->m_sectionIdStack.empty())
+            return;
+
         this->m_sectionIdStack.pop_back();
     }
 
@@ -930,6 +875,19 @@ namespace pl::core {
         return this->m_sectionIdStack.back();
     }
 
+    u64 Evaluator::getUserSectionId() const {
+        if (this->m_sectionIdStack.empty())
+            return 0;
+
+        for (auto it = this->m_sectionIdStack.rbegin(); it != this->m_sectionIdStack.rend(); ++it) {
+            if (*it != ptrn::Pattern::MainSectionId && *it != ptrn::Pattern::HeapSectionId && *it != ptrn::Pattern::PatternLocalSectionId && *it != ptrn::Pattern::InstantiationSectionId)
+                return *it;
+        }
+
+        return 0;
+    }
+
+
     u64 Evaluator::createSection(const std::string &name) {
         auto id = this->m_sectionId;
         this->m_sectionId++;
@@ -986,14 +944,20 @@ namespace pl::core {
         this->m_scopes.clear();
         this->m_callStack.clear();
         this->m_heap.clear();
-        this->m_patternLocalStorage.clear();
+
         this->m_templateParameters.clear();
+        this->m_currentTemplateArguments.clear();
+        this->m_typeTemplateParameters.clear();
+        this->m_attributedPatterns.clear();
+
+        this->m_patternLocalStorage.clear();
+
         this->m_stringPool.clear();
 
         this->m_mainResult.reset();
         this->m_aborted = false;
         this->m_evaluated = false;
-        this->m_attributedPatterns.clear();
+        this->m_shouldPauseNextLine = false;
 
         this->setPatternColorPalette(DefaultPatternColorPalette);
 
@@ -1057,55 +1021,73 @@ namespace pl::core {
                         if (localVariable)
                             this->pushSectionId(ptrn::Pattern::HeapSectionId);
 
+                        std::vector<std::shared_ptr<ptrn::Pattern>> patterns;
 
-                        for (auto &pattern : varDeclNode->createPatterns(this)) {
-                            if (localVariable) {
-                                auto name = pattern->getVariableName();
-                                wolv::util::unused(varDeclNode->execute(this));
+                        ON_SCOPE_EXIT {
+                            for (auto &pattern : patterns) {
+                                if (localVariable) {
+                                    auto name = pattern->getVariableName();
+                                    wolv::util::unused(varDeclNode->execute(this));
 
-                                this->setBitwiseReadOffset(startOffset);
-                            } else {
-                                this->m_patterns.push_back(std::move(pattern));
+                                    this->setBitwiseReadOffset(startOffset);
+                                } else {
+                                    this->m_patterns.push_back(std::move(pattern));
+                                }
+
+                                if (this->getCurrentControlFlowStatement() == ControlFlowStatement::Return)
+                                    break;
                             }
 
-                            if (this->getCurrentControlFlowStatement() == ControlFlowStatement::Return)
-                                break;
-                        }
+                            {
+                                auto name = varDeclNode->getName();
+                                if (varDeclNode->isInVariable() && this->m_inVariables.contains(name))
+                                    this->setVariable(name, this->m_inVariables[name]);
+                            }
 
-                        {
-                            auto name = varDeclNode->getName();
-                            if (varDeclNode->isInVariable() && this->m_inVariables.contains(name))
-                                this->setVariable(name, this->m_inVariables[name]);
-                        }
+                            if (localVariable)
+                                this->popSectionId();
+                        };
+
+                        varDeclNode->createPatterns(this, patterns);
 
-                        if (localVariable)
-                            this->popSectionId();
                     } else if (auto arrayVarDeclNode = dynamic_cast<ast::ASTNodeArrayVariableDecl *>(node); arrayVarDeclNode != nullptr) {
                         bool localVariable = arrayVarDeclNode->getPlacementOffset() == nullptr;
 
                         if (localVariable)
                             this->pushSectionId(ptrn::Pattern::HeapSectionId);
 
-                        for (auto &pattern : arrayVarDeclNode->createPatterns(this)) {
-                            if (localVariable) {
-                                wolv::util::unused(arrayVarDeclNode->execute(this));
+                        std::vector<std::shared_ptr<ptrn::Pattern>> patterns;
 
-                                this->setBitwiseReadOffset(startOffset);
-                            } else {
-                                this->m_patterns.push_back(std::move(pattern));
+                        ON_SCOPE_EXIT {
+                            for (auto &pattern : patterns) {
+                                if (localVariable) {
+                                    wolv::util::unused(arrayVarDeclNode->execute(this));
+
+                                    this->setBitwiseReadOffset(startOffset);
+                                } else {
+                                    this->m_patterns.push_back(std::move(pattern));
+                                }
                             }
-                        }
 
-                        if (localVariable)
-                            this->popSectionId();
+                            if (localVariable)
+                                this->popSectionId();
+                        };
+
+                        arrayVarDeclNode->createPatterns(this, patterns);
                     } else if (auto pointerVarDecl = dynamic_cast<ast::ASTNodePointerVariableDecl *>(node); pointerVarDecl != nullptr) {
-                        for (auto &pattern : pointerVarDecl->createPatterns(this)) {
-                            if (pointerVarDecl->getPlacementOffset() == nullptr) {
-                                err::E0003.throwError("Pointers cannot be used as local variables.");
-                            } else {
-                                this->m_patterns.push_back(std::move(pattern));
+                        std::vector<std::shared_ptr<ptrn::Pattern>> patterns;
+
+                        ON_SCOPE_EXIT {
+                            for (auto &pattern : patterns) {
+                                if (pointerVarDecl->getPlacementOffset() == nullptr) {
+                                    err::E0003.throwError("Pointers cannot be used as local variables.");
+                                } else {
+                                    this->m_patterns.push_back(std::move(pattern));
+                                }
                             }
-                        }
+                        };
+
+                        pointerVarDecl->createPatterns(this, patterns);
                     } else if (auto controlFlowStatement = dynamic_cast<ast::ASTNodeControlFlowStatement *>(node); controlFlowStatement != nullptr) {
                         this->pushSectionId(ptrn::Pattern::HeapSectionId);
                         auto result = node->execute(this);
@@ -1141,18 +1123,9 @@ namespace pl::core {
             }
         } catch (err::EvaluatorError::Exception &e) {
 
-            auto location = e.getUserData();
+            const auto location = e.getUserData();
 
-            this->getConsole().setHardError(err::PatternLanguageError(e.format(location),
-                location.line, location.column));
-
-            // Don't clear patterns on error if debug mode is enabled
-            if (this->m_debugMode)
-                return false;
-
-            this->m_patterns.clear();
-
-            this->m_currPatternCount = 0;
+            this->getConsole().setHardError(err::PatternLanguageError(e.format(location), location.line, location.column, this->getReadOffset()));
 
             return false;
         }
@@ -1182,7 +1155,8 @@ namespace pl::core {
                         evaluator->m_lastPauseLine = std::nullopt;
                 }
             }
-            evaluator->m_callStack.push_back(node->clone());
+            this->node = node;
+            this->offset = evaluator->getReadOffset();
         }
     }
 
@@ -1190,12 +1164,9 @@ namespace pl::core {
         if (evaluator->m_evaluated)
             return;
 
-        // Don't pop scopes if an exception is currently being thrown so we can generate
-        // a stack trace
-        if (std::uncaught_exceptions() > 0)
-            return;
-
-        evaluator->m_callStack.pop_back();
+        if (std::uncaught_exceptions() > 0) [[unlikely]]
+            if (node != nullptr)
+                evaluator->m_callStack.emplace_back(node->clone(), offset);
     }
 
     Evaluator::UpdateHandler Evaluator::updateRuntime(const ast::ASTNode *node) {
